@@ -16,6 +16,8 @@ DEFAULT_KEY_NAME = 'kommandos-key'
 
 DEFAULT_DNS_TTL = 86400
 
+KOMMANDOS_HOME_FOLDER = f'{os.path.expanduser("~")}/.aws-kommandos'
+
 if 'win' in sys.platform:
     print("Windows ain't supported")
     exit()
@@ -31,7 +33,8 @@ def get_arguments():
                              required=False,
                              type=str,
                              help="Specify the AWS access key ID to use. "
-                                  "If omitted, the script grabs the key ID from the AWS_ACCESS_KEY_ID env variable. "
+                                  "If omitted, the script grabs the key ID "
+                                  "from the AWS_ACCESS_KEY_ID env variable. "
                                   "If that variable does not present, the script takes default credentials "
                                   "located in the ~/.aws/credentials file.")
     credentials.add_argument('--access-key-secret',
@@ -39,7 +42,8 @@ def get_arguments():
                              required=False,
                              type=str,
                              help="Specify the AWS access key secret to use. "
-                                  "If omitted, the script grabs the key secret from the AWS_ACCESS_KEY_SECRET env variable. "
+                                  "If omitted, the script grabs the key secret "
+                                  "from the AWS_ACCESS_KEY_SECRET env variable. "
                                   "If that variable does not present, the script takes default credentials "
                                   "located in the ~/.aws/credentials file.")
     credentials.add_argument('--region-name',
@@ -65,9 +69,21 @@ def get_arguments():
                                    required=False,
                                    help="Provide this flag to disable API termination of the new instance. "
                                         "You would need to manually login to the AWS console "
-                                        "for disabling the instance. "
+                                        "for shutting down the instance. "
                                         "Useful if you wanna ensure you won't accidentally terminate "
-                                        "the required instance.")
+                                        "something you may need in the future.")
+    starting_commands.add_argument('--poll-ssh',
+                                   dest='poll_ssh',
+                                   action='store_true',
+                                   required=False,
+                                   help="Specify this flag if you want to poll the SSH service on remote "
+                                        "after the server boot. "
+                                        "The netcat tool needs to be installed on the system to use that. "
+                                        "Keep in mind that you should provide this flag in case you wanna "
+                                        "trigger script invocation on remote, "
+                                        "since Kommandos needs to know when the service is available for invoking it. "
+                                        "Also make sure an inbound rule in the corresponding security group "
+                                        "for connecting to the 22/tcp port.")
     starting_commands.add_argument('--link-fqdn',
                                    action='store_true',
                                    dest='link_fqdn',
@@ -268,7 +284,9 @@ def get_arguments():
                           default=DEFAULT_KEY_NAME,
                           type=str,
                           help="Specify the key pair name to use alongside the created instance. "
-                               f"Default is '{DEFAULT_KEY_NAME}'.")
+                               f"Default is '{DEFAULT_KEY_NAME}'. "
+                               f"The keys managed by Kommandos "
+                               f"are stored under the '{KOMMANDOS_HOME_FOLDER}' directory.")
     ssh_keys.add_argument("--force-recreate-key",
                           dest="force_recreate_key",
                           action='store_true',
@@ -327,8 +345,12 @@ def get_arguments():
         if not options.security_group_id:
             parser.error('The --security-group-id arg cannot be blank when requesting a new instance. '
                          'Use --help for more info.')
-        if options.invoke_script and not os.path.exists(options.invoke_script):
-            parser.error("The script file you're providing with --invoke-script doesn't seem to exist")
+        if options.invoke_script:
+            if not os.path.exists(options.invoke_script):
+                parser.error("The script file you're providing with --invoke-script doesn't seem to exist")
+            if not options.poll_ssh:
+                parser.error('The --poll-ssh argument must be given if you require remote script invocation. '
+                             'Use --help for more info.')
     else:
         if options.invoke_script:
             parser.error("Invoking scripts on remote is only supported while creating new instances (at least yet)")
@@ -387,16 +409,22 @@ class AwsManager:
                  aws_access_key_id: str = None,
                  aws_access_key_secret: str = None,
                  region_name: str = None):
+        self.home_folder = KOMMANDOS_HOME_FOLDER
         try:
             self.ec2 = boto3.resource('ec2', aws_access_key_id=aws_access_key_id,
-                                      aws_access_key_secret=aws_access_key_secret,
+                                      aws_secret_access_key=aws_access_key_secret,
                                       region_name=region_name)
             self.route53_client = boto3.client('route53', aws_access_key_id=aws_access_key_id,
-                                               aws_access_key_secret=aws_access_key_secret,
+                                               aws_secret_access_key=aws_access_key_secret,
                                                region_name=region_name)
             self.ec2_client = boto3.client('ec2', aws_access_key_id=aws_access_key_id,
-                                           aws_access_key_secret=aws_access_key_secret,
+                                           aws_secret_access_key=aws_access_key_secret,
                                            region_name=region_name)
+
+            if not os.path.exists(self.home_folder):
+                os.makedirs(self.home_folder)
+                print(f"The Kommandos's home folder has been created at '{self.home_folder}'. "
+                      f"The SSH private keys that are to be created by Kommandos will be stored there.")
         except Exception as e:
             print(f"Kommandos initialization failed: {e}")
             exit(1)
@@ -505,14 +533,20 @@ class AwsManager:
         if key_pairs:
             print('The SSH key pairs are:')
             for kp in key_pairs:
+                key_name = kp['KeyName']
                 key = {
                     'KeyPairId': kp['KeyPairId'],
-                    'KeyName': kp['KeyName']
+                    'KeyName': key_name
                 }
                 if verbose:
                     key['KeyFingerprint'] = kp['KeyFingerprint']
                     if 'Tags' in kp and kp['Tags']:
                         key['Tags'] = kp['Tags']
+                if os.path.exists(f"{self.home_folder}/{key_name}.pem"):
+                    key_local_path = f"{self.home_folder}/{key_name}.pem"
+                else:
+                    key_local_path = '-'
+                key['KeyLocalPath'] = key_local_path
                 pprint(key, sort_dicts=False)
         else:
             print('There are no SSH key pairs')
@@ -524,12 +558,13 @@ class AwsManager:
         if response and 'KeyMaterial' in response:
             private_key = response['KeyMaterial']
             file_name = f"{key_name}.pem"
+            key_path = f"{self.home_folder}/{file_name}"
 
-            print(colored(f'Saving the private key to {file_name}', 'green'))
-            with open(file_name, 'w') as f:
+            print(colored(f'Saving the private key to {key_path}', 'green'))
+            with open(key_path, 'w') as f:
                 f.write(private_key)
             # 0x400 -r--------
-            os.chmod(file_name, 0o400)
+            os.chmod(key_path, 0o400)
 
     ### DELETE A KEY PAIR
     def delete_key_pair(self, key_name: str):
@@ -537,9 +572,10 @@ class AwsManager:
         self.ec2_client.delete_key_pair(KeyName=key_name)
 
         file_name = f"{key_name}.pem"
-        if os.path.exists(file_name):
-            os.chmod(file_name, 0o600)
-            os.remove(file_name)
+        key_path = f"{self.home_folder}/{file_name}"
+        if os.path.exists(key_path):
+            os.chmod(key_path, 0o600)
+            os.remove(key_path)
 
     ## SECURITY GROUPS
     ### GET SECURITY GROUPS
@@ -943,7 +979,8 @@ class AwsManager:
                      security_group_id: str,
                      instance_type: str,
                      instance_name: str,
-                     disable_api_termination: bool = False):
+                     disable_api_termination: bool = False,
+                     poll_ssh: bool = False):
         print(f'Starting a new instance: '
               f'{image_id} {key_pair_name} {security_group_id} {instance_type} {instance_name}')
         if disable_api_termination:
@@ -978,27 +1015,29 @@ class AwsManager:
             instance = self.get_instance(new_instance['InstanceId'])
             instance.wait_until_running()
             print(f'The server is up and running at {colored(instance.public_ip_address, "green")}')
-            print('Waiting until the SSH service is available...')
-            self.poll_ssh_status(instance.id)
+            if poll_ssh:
+                print('Waiting until the SSH service is available...')
+                self.poll_ssh_status(instance.id)
             identified_user_name = self.get_default_ami_user_name(instance.image_id)
+            key_path = f"{self.home_folder}/{key_pair_name}.pem"
             print("Use the following command for connecting to the instance: " +
-                  colored(f"ssh {identified_user_name}@{instance.public_ip_address} -i {key_pair_name}.pem", 'green'))
+                  colored(f"ssh {identified_user_name}@{instance.public_ip_address} -i {key_path}", 'green'))
             return instance
 
     def invoke_script(self, instance_id: str,
                       file_name: str,
                       parameters: list = None,
-                      key_pair_name: str = None):
+                      key_pair_path: str = None):
         instance = self.get_instance(instance_id=instance_id)
         ip_address = instance.public_ip_address
         username = self.get_default_ami_user_name(image_id=instance.image_id)
         print(colored(f"Invoking the {file_name} script on the instance hosted at {ip_address}", 'yellow'))
 
-        if not key_pair_name:
-            key_pair_name = f"{instance.key_pair.name}.pem"
+        if not key_pair_path:
+            key_pair_path = f"{self.home_folder}/{instance.key_pair.name}.pem"
 
-        if not os.path.exists(key_pair_name):
-            raise Exception(f"The RSA private key '{key_pair_name}' has not been found at the given path")
+        if not os.path.exists(key_pair_path):
+            raise Exception(f"The RSA private key '{key_pair_path}' has not been found at the given path")
 
         if parameters:
             print(colored(f"Script parameters are: {parameters}", 'yellow'))
@@ -1026,10 +1065,10 @@ class AwsManager:
             remote_script_name = f"{remote_script_folder}/{script_name}"
 
         subprocess.call(['scp', '-o', 'StrictHostKeyChecking=accept-new',
-                         '-i', key_pair_name,
+                         '-i', key_pair_path,
                          script_name, f"{username}@{ip_address}:{remote_script_name}"])
         subprocess.call(['ssh', '-o', 'StrictHostKeyChecking=accept-new',
-                         '-i', key_pair_name,
+                         '-i', key_pair_path,
                          f"{username}@{ip_address}",
                          "/bin/bash", remote_script_name])
         print(colored(f"The '{file_name}' script has been invoked as {username}@{ip_address}", 'yellow'))
@@ -1170,7 +1209,8 @@ def main():
                                                 security_group_id=security_group_id,
                                                 instance_type=options.instance_type,
                                                 instance_name=instance_name,
-                                                disable_api_termination=options.disable_api_termination)
+                                                disable_api_termination=options.disable_api_termination,
+                                                poll_ssh=options.poll_ssh)
         if options.link_fqdn:
             ttl = options.ttl
             aws_manager.create_dns_record(hosted_zone_name=domain_name,
